'
' Generic toolbar using QPanel & QCoolBtns v0.7
' 
' Notes:
' 1.    User code can $DEFINE MAXTOOLBTNS to override value below
' 2.    ToolBar Align property: alNone & alClient overridden to alTop
' 3.    BtnSpacer is gap between buttons (does not alter the gap from each button
'       to its borders). Active for both horizontal and vertical ToolBars.
' 4.    Only ONE image per button presently
' 5.    BMP or BMPHandle MUST be set AFTER BtnCount is set!!!
' 6.    Multi-image buttons and multi-tabbed buttons coming.
'
$INCLUDE "QImageListEX.inc"                                             ' Build using a bmp "strip" image

$IFNDEF __RQINC2
$DEFINE alNone          0
$DEFINE alTop           1
$DEFINE alBottom        2
$DEFINE alLeft          3
$DEFINE alRight         4
$DEFINE alClient        5
$ENDIF

$IFNDEF True
$DEFINE False           0
$DEFINE True            1
$ENDIF

$IFNDEF MAXTOOLBTNS
$DEFINE MAXTOOLBTNS     64                                              ' User code settable - cannot use REDIM!
$ENDIF

DIM ButtonImages AS QImageListEX                                        ' MUST be global!

DECLARE SUB ToolBtnClick_EventTemplate (Index AS integer)

TYPE QToolBarEX EXTENDS QPANEL
WITH QToolBarEX

    BtnCount        AS integer PROPERTY SET SetBtnCount                 ' Number of button images in bitmap "strip"
    BtnSpacer       AS integer PROPERTY SET SetBtnSpacing               ' Spacing between buttons
    BtnMask         AS integer PROPERTY SET SetBtnMask                  ' Defaults to bottom LHS pixel (of each image!)
    BMP             AS string  PROPERTY SET SetBtnFile                  ' Load a bitmap file
    BMPHandle       AS integer PROPERTY SET SetBtnResource              ' Load a bitmap resource
    Hint            AS string  PROPERTY SET SetHint
    Cursor          AS integer PROPERTY SET SetCursor

    OnSelect        AS EVENT(ToolBtnClick_EventTemplate)

PRIVATE:
    tbButtons(MAXTOOLBTNS)    AS QCOOLBTN                               ' Unable to "REDIM" on the fly!!
    BtnWidth        AS integer
    BtnHeight       AS integer
'    imgWidth        AS integer                                          ' Multi-image stuff
'    imgHeight       AS integer                                          ' ditto
'    InitMask        AS integer                                          ' Attempt some logic for init??

    SUB WhichButton(Sender AS QCOOLBTN)
    DEFINT i
        i=Sender.Tag
        IF .OnSelect>0 THEN
            CALLFUNC(.OnSelect,i)
        END IF
    END SUB

    SUB InitButtons
    DEFINT i

        FOR i=0 TO .BtnCount-1
            .tbButtons(i).Parent=THIS
            .tbButtons(i).Left=1                                        ' Default - if necessary fix below
            .tbButtons(i).Top=1                                         ' ditto
            .tbButtons(i).Width=.BtnWidth
            .tbButtons(i).Height=.BtnHeight
            .tbButtons(i).NumBMPs=1
            .tbButtons(i).Spacing=0
            .tbButtons(i).Flat=True
            .tbButtons(i).Tag=i                                         ' Can use this, not available to user code
            .tbButtons(i).OnClick=THIS.WhichButton
            SELECT CASE .Align
                CASE alLeft, alRight
                    .tbButtons(i).Top=i*(.BtnHeight+.BtnSpacer)         ' Fix this now
                CASE alNone, alTop, alBottom
                    .tbButtons(i).Left=i*(.BtnWidth+.BtnSpacer)         ' ditto
            END SELECT
            .tbButtons(i).BMP=ButtonImages.GetBMP(i)                    ' Load button images
        NEXT i
        FOR i=i TO MAXTOOLBTNS
            .tbButtons(i).Enabled=False
            .tbButtons(i).Visible=False
        NEXT i
    END SUB

PUBLIC:
    PROPERTY SET SetBtnCount(numButtons AS integer)
        IF (numButtons>0) AND (numButtons<=MAXTOOLBTNS) THEN
            .BtnCount=numButtons
        END IF
    END PROPERTY

    PROPERTY SET SetBtnSpacing(gap AS integer)
        IF gap > 0 THEN
            .BtnSpacer=gap
        END IF
    END PROPERTY

    PROPERTY SET SetBtnMask(Mask AS integer)
        .BtnMask=Mask                                                   ' FIXME: Override default?
    END PROPERTY

    PROPERTY SET SetBtnFile(FileStr AS string)
        IF FileExists(FileStr)=0 THEN  ShowMessage "File error! Cannot load bitmap": EXIT SUB
        IF .BtnCount=0 THEN ShowMessage "Button count error! Cannot load bitmap": EXIT SUB

        IF .Align=alClient OR .Align=alNone THEN .Align=alTop           ' Not allowed default to top
        ButtonImages.FixWidth=0: ButtonImages.FixHeight=0               ' Default - no stretch
        SELECT CASE .Align
            CASE alLeft, alRight
                IF .Width>0 THEN
                    .BtnWidth=.Width-2
                    ButtonImages.FixWidth=.Width-4           ' Stretch image widths
                END IF
            CASE alTop, alBottom
                IF .Height>0 THEN
                    .BtnHeight=.Height-2
                    ButtonImages.FixHeight=.Height-4         ' Stretch image heights
                END IF
        END SELECT
        ButtonImages.ExtractFile(FileStr,.BtnCount,.BtnMask)
        IF ButtonImages.Count<>.BtnCount THEN
            ShowMessage "Unknown error! Failed to load bitmap"
        ELSE
            IF .Width=0 THEN
                .BtnWidth=ButtonImages.Width+2               ' Size from the bitmap images
                .Width=ButtonImages.Width+4
            END IF
            IF .Height=0 THEN
                .BtnHeight=ButtonImages.Height+2             ' Size from the bitmap images
                .Height=ButtonImages.Height+4
            END IF
            .InitButtons
        END IF
    END PROPERTY

    PROPERTY SET SetBtnResource(ResIndex AS integer)
        IF .BtnCount=0 THEN ShowMessage "Button count unknown! Cannot load bitmap": EXIT SUB

        IF .Align=alClient OR .Align=alNone THEN .Align=alTop           ' Not allowed default to top
        ButtonImages.FixWidth=0: ButtonImages.FixHeight=0               ' Default - no stretch
        SELECT CASE .Align
            CASE alLeft, alRight
                IF .Width>0 THEN
                    .BtnWidth=.Width-2
                    ButtonImages.FixWidth=.Width-4
                END IF
            CASE alTop, alBottom
                IF .Height>0 THEN
                    .BtnHeight=.Height-2
                    ButtonImages.FixHeight=.Height-4
                END IF
        END SELECT
        ButtonImages.ExtractRes(ResIndex,.BtnCount,.BtnMask)
        IF ButtonImages.Count<>.BtnCount THEN
            ShowMessage "Unknown error! Failed to load bitmap"
        ELSE
            IF .Width=0 THEN
                .Width=ButtonImages.Width+4
                .BtnWidth=ButtonImages.Width+2
            END IF
            IF .Height=0 THEN
                .Height=ButtonImages.Height+4
                .BtnHeight=ButtonImages.Height+2
            END IF
            .InitButtons
        END IF
    END PROPERTY

    PROPERTY SET SetCursor(curType AS integer)
    DEFINT i

        IF (curType >= -21) AND (curType <= 0) THEN
            FOR i=0 TO .BtnCount-1
                .tbButtons(i).Cursor=curType
            NEXT i
        END IF
    END PROPERTY

    ' Multi-hints CSV string. If button hint string present, then button hint is enabled 
    PROPERTY SET SetHint(hString AS string)
    DEFINT i, count

        IF hString="" THEN EXIT SUB
        count=TALLY(hString,",")+1
        FOR i=1 TO .BtnCount
            IF i>count THEN EXIT FOR
            .tbButtons(i-1).Hint=FIELD$(hString,",",i)
            .tbButtons(i-1).ShowHint=True
        NEXT i
        FOR i=i TO .BtnCount                                            ' Just in case!
            .tbButtons(i-1).ShowHint=False
        NEXT i
    END PROPERTY

    EVENT OnClick
    ' Do nothing on clicking the QPanel
    END EVENT

CONSTRUCTOR
    Align=alTop
    Left=0
    Top=0
    Width=0
    Height=0
'    imgWidth=0
'    imgHeight=0
    BtnCount=0
    BtnSpacer=1
    BtnMask=&hffffff
    Hint=""
    Cursor=0                                                            ' crDefault
    OnSelect=0                                                          ' Default no OnClick
END CONSTRUCTOR
END WITH
END TYPE
