'-------------------------------------------------------------------------------
' QMainMenu & QPopupMenu example code
'
' http://msdn.microsoft.com/en-us/library/windows/desktop/ms647987%28v=vs.85%29.aspx
' http://msdn.microsoft.com/en-us/library/windows/desktop/ms647616%28v=vs.85%29.aspx
'
' Basic framework code provided by RapidFRM v1.80
' NB One MainMenu per form only & ALL visible components may have a popup menu
' associated with it.
' Menu code has been added manually (not provided through RapidFRM). 
' Each block inserted has a comment line starting with #MENU# or #POPUP#
' for ease of identification/location.
' The Open dialog is the newer 'explorer' style, the save dialog is the older style
' (set by a property). Currently older style can be positioned, but not newer style.
'-------------------------------------------------------------------------------
#INCLUDE "windows.bi"
#INCLUDE "win/commctrl.bi"
InitCommonControls()

#INCLUDE "win/RQInclude.bi"

' #POPUP# MenuItems IDs values starting at 1 (for convenience!)
ENUM FRM_POPUP
    TEXT_COPY = 1
    TEXT_CUT
    TEXT_PASTE
    TEXT_SEL_ALL
    POPUP_ABOUT
END ENUM

DECLARE SUB PopupMenuSelect(Index AS integer, Sender AS QFORM)

' #MENU# MenuItems IDs values starting at 1 (for convenience!)
ENUM MENUID_ENUM
    MENUID_FILE_OPEN = 1
    MENUID_FILE_SAVE
    MENUID_FILE_EXIT
    MENUID_FONT_DLG
    MENUID_COLOR_DLG
    MENUID_PRNSET_DLG
    MENUID_PRINT_DLG
    MENUID_EDIT_CUT
    MENUID_EDIT_COPY
    MENUID_EDIT_PASTE
    MENUID_EDIT_UNDO
    MENUID_EDIT_REDO
    MENUID_SEARCH_FINDPREV
    MENUID_SEARCH_FINDNEXT
    MENUID_SEARCH_REPLACE
    MENUID_ABOUT
END ENUM

DECLARE SUB Menu_Select(Index AS integer)
DECLARE SUB TestOpenDialog()
DECLARE SUB TestSaveDialog()
DECLARE SUB TestFontDialog()
DECLARE SUB TestColorDialog()
DECLARE SUB TestPageSetup()
DECLARE SUB TestPrintDialog()
DECLARE SUB ShowAbout()
'-------------------------------------------------------------------------------
' Global data from Form designer sheet
'-------------------------------------------------------------------------------
DIM SHARED AS QFORM Form1
DIM SHARED AS QFORM PTR lpForm1 = @Form1
DIM SHARED AS QBUTTON Butn1
DIM SHARED AS QBUTTON PTR lpButn1 = @Butn1
DIM SHARED AS QLABEL Label1
DIM SHARED AS QLABEL PTR lpLabel1 = @Label1

'-------------------------------------------------------------------------------
' Forward declarations for component events 
'-------------------------------------------------------------------------------
DECLARE SUB Done()

'-------------------------------------------------------------------------------
' Configure components
'-------------------------------------------------------------------------------
SUB ConfigureComponents

    lpForm1->ObjIndex = 0
    lpForm1->Caption = "Menu Test Code"
    lpForm1->OnPopup = CAST(ANY PTR, @PopupMenuSelect)

    lpButn1->ObjIndex = 1
    lpButn1->Left = (lpForm1->Width-lpButn1->Width)\2
    lpButn1->Top = 145
    lpButn1->Caption = "EXIT"
    lpButn1->OnClick = CAST(ANY PTR, @Done)

    lpLabel1->ObjIndex = 2
    lpLabel1->Left = 10
    lpLabel1->Top = 30
    lpLabel1->Width = lpForm1->Width-20
    lpLabel1->Alignment = taCenter
    lpLabel1->Font.Color = &hff
    lpLabel1->Caption = "This is a label string"

END SUB

'-------------------------------------------------------------------------------
' Create components
'-------------------------------------------------------------------------------
' #MENU# 
SUB CreateMainMenu
DIM AS HMENU mnuMain
DIM AS HMENU mnuFile, mnuDlg, mnuEdit, mnuSearch, mnuFind, mnuHelp

    mnuMain = CreateMenu()                                                      ' Create the main menu handle
    mnuFile = CreateMenu(): mnuDlg = CreateMenu(): mnuEdit = CreateMenu()       ' Now for the sub-menus
    mnuSearch = CreateMenu(): mnuFind = CreateMenu(): mnuHelp = CreateMenu()

    ' NB Parm 2 (0 = MF_ENABLED!!)
    ' Create the main menu items
    InsertMenu(mnuMain, 0, MF_POPUP, CINT(mnuFile), "&File")
    InsertMenu(mnuMain, 0, MF_POPUP, CINT(mnuDlg), "&Dialogs")
    InsertMenu(mnuMain, 0, MF_POPUP, CINT(mnuEdit), "&Edit")
    InsertMenu(mnuMain, 0, MF_POPUP, CINT(mnuSearch), "Sear&ch")
    InsertMenu(mnuMain, 0, MF_POPUP, CINT(mnuHelp), "&Help")
    ' SubItem #1 File options
    AppendMenu(mnuFile, 0, MENUID_FILE_OPEN, "&Open")
    AppendMenu(mnuFile, 0, MENUID_FILE_SAVE, "&Save As")
    AppendMenu(mnuFile, MF_MENUBARBREAK, 0, 0)                                ' separator
    AppendMenu(mnuFile, 0, MENUID_FILE_EXIT, "E&xit")
    ' SubItem #2 Dialog options
    AppendMenu(mnuDlg, 0, MENUID_FONT_DLG, "&Font")
    AppendMenu(mnuDlg, 0, MENUID_COLOR_DLG, "&Color")
    AppendMenu(mnuDlg, 0, MENUID_PRNSET_DLG, "Pa&ge Setup")
    AppendMenu(mnuDlg, 0, MENUID_PRINT_DLG, "&Print")
    ' SubItem #3 Edit options
    AppendMenu(mnuEdit, 0, MENUID_EDIT_CUT, "&Cut")
    AppendMenu(mnuEdit, 0, MENUID_EDIT_COPY, "C&opy")
    AppendMenu(mnuEdit, MF_DISABLED OR MF_GRAYED, MENUID_EDIT_PASTE, "&Paste")
    AppendMenu(mnuEdit, MF_MENUBARBREAK, 0, 0)
    AppendMenu(mnuEdit, 0, MENUID_EDIT_UNDO, "&Undo")
    AppendMenu(mnuEdit, 0, MENUID_EDIT_REDO, "&Redo")
    ' SubItem #4 Search options with subItems
    InsertMenu(mnuSearch, 0, MF_POPUP, CINT(mnuFind), "&Find")
    AppendMenu(mnuFind, 0, MENUID_SEARCH_FINDPREV, "Find &Prev")                ' Sub item to mnuFind
    AppendMenu(mnuFind, 0, MENUID_SEARCH_FINDNEXT, "Find &Next")                ' Sub item to mnuFind
    AppendMenu(mnuSearch, MF_CHECKED, MENUID_SEARCH_REPLACE, "&Replace")
    ' SubItem #5
    AppendMenu(mnuHelp, 0, MENUID_ABOUT, "&About")

    SetMenu(lpForm1->Handle, mnuMain)                                           ' Set the association to the main form
    DrawMenuBar(lpForm1->Handle)                                                ' MUST be done last thing
END SUB

SUB CreatePopupMenus
' #POPUP# This popup menu associated with the main form
' NB AppendMenu() Parm 2 (0 = MF_ENABLED)
    lpForm1->PopupMenu=CreatePopupMenu()                                        ' Create an empty menu
    AppendMenu(lpForm1->PopupMenu, 0, TEXT_COPY, "Copy")
    AppendMenu(lpForm1->PopupMenu, 0, TEXT_CUT, "Cut")
    AppendMenu(lpForm1->PopupMenu, 0, TEXT_PASTE, "Paste")
    AppendMenu(lpForm1->PopupMenu, 0, 0, 0)                                     ' separator
    AppendMenu(lpForm1->PopupMenu, 0, TEXT_SEL_ALL, "Select All")
    AppendMenu(lpForm1->PopupMenu, 0, 0, 0)                                     ' separator
    AppendMenu(lpForm1->PopupMenu, 0, POPUP_ABOUT, "About")
END SUB

SUB CreateComponents
DIM AS UINT StdStyle, ExtStyle=0
DIM AS string ObjCaption

    lpForm1->PreSetConfig()
    StdStyle=lpForm1->objStyle
    IF lpForm1->BorderStyle > bsDialog THEN
        ExtStyle=WS_EX_TOOLWINDOW
    END IF
    lpForm1->Handle=CreateWindowEx(ExtStyle, "RQClass", lpForm1->Caption, StdStyle, _
                    lpForm1->Left, lpForm1->Top, lpForm1->Width, lpForm1->Height, _
                    0, 0, Application.hInstance, 0)
    Application.hMainForm=lpForm1->Handle
    lpForm1->PostConfig()

    lpButn1->PreSetConfig()
    ObjCaption=lpButn1->Caption
    StdStyle=lpButn1->objStyle
    IF lpButn1->Visible THEN StdStyle OR= WS_VISIBLE
    ExtStyle=0

    lpButn1->Handle=CreateWindowEx(ExtStyle,lpButn1->objClass, ObjCaption, WS_CHILD OR StdStyle, _
                    lpButn1->Left, lpButn1->Top, lpButn1->Width, lpButn1->Height, _
                    lpForm1->Handle, CAST(HMENU, lpButn1->objIndex), 0, 0)

    lpButn1->Parent = lpForm1->Handle
    lpButn1->PostConfig()
    EnableWindow(lpButn1->Handle, lpButn1->Enabled)

    lpLabel1->PreSetConfig()
    ObjCaption=lpLabel1->Caption
    lpLabel1->Parent=lpForm1->Handle
    StdStyle=lpLabel1->objStyle
    IF lpLabel1->Visible THEN StdStyle += WS_VISIBLE
    ExtStyle=0

    lpLabel1->Handle=CreateWindowEx(ExtStyle,lpLabel1->objClass, ObjCaption, WS_CHILD OR StdStyle, _
                    lpLabel1->Left, lpLabel1->Top, lpLabel1->Width, lpLabel1->Height, _
                    lpForm1->Handle, CAST(HMENU, lpLabel1->objIndex), 0, 0)

    lpLabel1->Parent = lpForm1->Handle
    lpLabel1->PostConfig()
    EnableWindow(lpLabel1->Handle, lpLabel1->Enabled)

    CreateMainMenu()
    CreatePopupMenus()
END SUB

'-------------------------------------------------------------------------------
' Remove Timers on Exit
'-------------------------------------------------------------------------------
SUB KillAllTimers
    KillTimer(Application.hMainForm, SYS_TIMER_IDX)
END SUB

'-------------------------------------------------------------------------------
' System timer - Display hints
'-------------------------------------------------------------------------------
SUB SysDisplayHint(Parent AS HWND, HintString AS string)
DIM AS TOOLINFO ti

    ti.cbSize=sizeof(ti)
    ti.uFlags=TTF_SUBCLASS: ti.hwnd=Parent
    ti.hinst=NULL: ti.lpszText=STRPTR(HintString)
    GetClientRect(Parent, @ti.rect)
    SendMessage(Application.hWndTTip, TTM_ADDTOOL, 0, CAST(LPARAM, @ti))	
END SUB

SUB CheckSystemTimers
STATIC AS HWND TT_LastHWND
DIM AS HWND TT_hWnd
DIM AS Point TT_pnt

    TT_pnt.X=Screen.MouseX: TT_pnt.Y=Screen.MouseY
    TT_hWnd=WindowFromPoint(TT_pnt)
    IF TT_LastHWND<>TT_hWnd THEN
        TT_LastHWND=TT_hWnd
        SELECT CASE TT_hWnd
            CASE lpForm1->Handle
                IF lpForm1->ShowHint THEN
                    SysDisplayHint(lpForm1->Handle, lpForm1->Hint)
                END IF
            CASE lpButn1->Handle
                IF lpButn1->ShowHint THEN
                    SysDisplayHint(lpButn1->Handle, lpButn1->Hint)
                END IF
            CASE lpLabel1->Handle
                IF lpLabel1->ShowHint THEN
                    SysDisplayHint(lpLabel1->Handle, lpLabel1->Hint)
                END IF
        END SELECT
    END IF
END SUB

'-------------------------------------------------------------------------------
' Start of Main WndProc code
'-------------------------------------------------------------------------------

SUB CheckWmTimers(hWnd AS HWND, wMsg AS UINT, wParam AS WPARAM, lParam AS LPARAM)
    SELECT CASE wParam
        CASE App_Timer.ObjIndex:  CheckSystemTimers
    END SELECT
END SUB

SUB CheckWmCommandMsg(wParam AS WPARAM, lParam AS LPARAM)
    SELECT CASE LOWORD(wParam)
        CASE lpButn1->ObjIndex
            IF lpButn1->OnClick>0 THEN lpButn1->OnClick(lpButn1)
        CASE lpLabel1->ObjIndex
            IF lpLabel1->OnClick>0 THEN lpLabel1->OnClick(lpLabel1)
    END SELECT
END SUB

SUB CheckWmNotifyMsg(wParam AS WPARAM, lParam AS LPARAM)
END SUB

SUB CheckWmPaintMsg(hWnd AS HWND, wMsg AS UINT, wParam AS WPARAM, lParam AS LPARAM)
    IF lpForm1->OnPaint>0 THEN lpForm1->OnPaint(lpForm1)
END SUB

FUNCTION CheckBtnColor(wParam AS WPARAM, lParam AS LPARAM) AS LRESULT
DIM AS HBRUSH hBrush=NULL
' Only for OwnerDraw buttons
    RETURN CAST(LRESULT, hBrush)
END FUNCTION

FUNCTION CheckStaticColor(wParam AS WPARAM, lParam AS LPARAM) AS LRESULT
DIM AS HBRUSH hBrush=NULL
DIM AS HDC hDC=CAST(HDC, wParam)
DIM AS HWND hWnd=CAST(HWND, lParam)

    SELECT CASE hWnd
        CASE lpLabel1->Handle
            SetBKMode(hDC, 1)
            SetTextColor(hDC, CAST(COLORREF, lpLabel1->Font.Color))
            hBrush=CreateSolidBrush(CAST(COLORREF, lpLabel1->Color))
    END SELECT
    RETURN CAST(LRESULT, hBrush)
END FUNCTION

FUNCTION CheckEditColor(wParam AS WPARAM, lParam AS LPARAM) AS LRESULT
DIM AS HBRUSH hBrush=NULL
    RETURN CAST(LRESULT, hBrush)
END FUNCTION

FUNCTION CheckListBoxColor(wParam AS WPARAM, lParam AS LPARAM) AS LRESULT
DIM AS HBRUSH hBrush=NULL
    RETURN CAST(LRESULT, hBrush)
END FUNCTION

SUB CheckWmObjScroll(hWnd AS HWND, wMsg AS UINT, wParam AS WPARAM, lParam AS LPARAM)
END SUB

SUB CheckSocketEvent(hWnd AS HWND, wMsg AS UINT, wParam AS WPARAM, lParam AS LPARAM)
END SUB

' #POPUP# Hook into the Popup menus for all of your components here
SUB CheckWmContextMenu(hWnd AS HWND, wMsg AS UINT, wParam AS WPARAM, lParam AS LPARAM)
    SELECT CASE wParam
        CASE lpForm1->Handle
            IF lpForm1->OnPopup>0 THEN
                DIM AS UINT popFlags=TPM_RETURNCMD OR TPM_RIGHTBUTTON OR TPM_NOANIMATION
                DIM AS integer Result, x=LOWORD(lParam), y=HIWORD(lParam)
                Result=TrackPopupMenuEx(lpForm1->PopupMenu, popFlags, x, y, lpForm1->Handle, 0)
                IF Result>0 THEN lpForm1->OnPopup(Result, lpForm1)
            END IF
    END SELECT
END SUB

SUB CheckWmHitTest(hWnd AS HWND, wMsg AS UINT, wParam AS WPARAM, lParam AS LPARAM)
END SUB

SUB CheckWmDrawItem(wParam AS WPARAM, lParam AS LPARAM)
END SUB

FUNCTION MainWndProc(hWnd AS HWND, wMsg AS UINT, wParam AS WPARAM, lParam AS LPARAM) AS LRESULT
STATIC AS integer Sizing
    
    SELECT CASE wMsg
        CASE WM_CREATE
            Sizing=False

        CASE WM_COMMAND
            IF lParam<>0 THEN
                CheckWmCommandMsg(wParam, lParam)
            ELSE
                IF HIWORD(wParam)=0 THEN                                ' Menu msg?
                    Menu_Select(LOWORD(wParam))                         ' Call menu sub
                    RETURN 0
                ELSE
                    ' Accelerator message code goes here...
                END IF
            END IF

        CASE WM_NOTIFY
            CheckWmNotifyMsg(wParam, lParam)

        CASE WM_PAINT
            CheckWmPaintMsg(hWnd, wMsg, wParam, lParam)

        CASE WM_TIMER
            CheckWmTimers(hWnd, wMsg, wParam, lParam)

        CASE WM_NCHITTEST
            CheckWmHitTest(hWnd, wMsg, wParam, lParam)

        CASE WM_HSCROLL, WM_VSCROLL
            IF lParam<>0 THEN CheckWmObjScroll(hWnd, wMsg, wParam, lParam)

        CASE WM_CTLCOLORBTN
            RETURN CheckBtnColor(wParam, lParam)

        CASE WM_CTLCOLOREDIT
            RETURN CheckEditColor(wParam, lParam)

        CASE WM_CTLCOLORSTATIC
            RETURN CheckStaticColor(wParam, lParam)

        CASE WM_CTLCOLORLISTBOX
            RETURN CheckListBoxColor(wParam, lParam)

        CASE WM_DRAWITEM
            CheckWmDrawItem(wParam, lParam)

        CASE WM_CONTEXTMENU
            CheckWmContextMenu(hWnd, wMsg, wParam, lParam)

        CASE WM_SOCK_NOTIFY
            CheckSocketEvent(hWnd, wMsg, wParam, lParam)

        CASE WM_MOUSEMOVE
            IF lpForm1->Cursor<>crDefault THEN
                DIM AS HCURSOR oldCursor=SetCursor(lpForm1->hCursor)
            END IF
            IF lpForm1->OnMouseMove>0 THEN
                lpForm1->OnMouseMove(LOWORD(lParam), HIWORD(lParam), 0, lpForm1)
            END IF

        CASE WM_LBUTTONDBLCLK
            IF lpForm1->OnClick>0 THEN lpForm1->OnClick(lpForm1)

        CASE WM_LBUTTONDOWN
            IF lpForm1->OnMouseDown>0 THEN
                lpForm1->OnMouseDown(MK_LBUTTON, LOWORD(lParam), HIWORD(lParam), 0, lpForm1)
            END IF

        CASE WM_LBUTTONUP
            IF lpForm1->OnMouseUp>0 THEN
                lpForm1->OnMouseUp(MK_LBUTTON, LOWORD(lParam), HIWORD(lParam), 0, lpForm1)
            END IF
            IF lpForm1->OnClick>0 THEN lpForm1->OnClick(lpForm1)

        CASE WM_RBUTTONDOWN
            IF lpForm1->OnMouseDown>0 THEN
                lpForm1->OnMouseDown(MK_RBUTTON, LOWORD(lParam), HIWORD(lParam), 0, lpForm1)
            END IF

        CASE WM_RBUTTONUP
            IF lpForm1->OnMouseUp>0 THEN
                lpForm1->OnMouseUp(MK_RBUTTON, LOWORD(lParam), HIWORD(lParam), 0, lpForm1)
            END IF

        CASE WM_KEYDOWN
            IF lpForm1->OnKeyDown>0 THEN lpForm1->OnKeyDown(LOWORD(wParam), 0, lpForm1)

        CASE WM_KEYUP
            IF lpForm1->OnKeyUp>0 THEN lpForm1->OnKeyUp(LOWORD(wParam), 0, lpForm1)

        CASE WM_CHAR
            IF lpForm1->OnKeyPress>0 THEN lpForm1->OnKeyPress(LOBYTE(wParam), lpForm1)

        CASE WM_SYSCOMMAND
            IF (wParam AND &hfff0) = SC_CLOSE THEN
                DIM AS integer Action=1                                      ' Default Action is quit
                IF lpForm1->OnClose>0 THEN lpForm1->OnClose(Action, lpForm1) ' Action passed by reference so
                IF Action=0 THEN RETURN 0                                    ' user can change default action
            END IF

        CASE WM_ENTERSIZEMOVE
            Sizing=True

        CASE WM_EXITSIZEMOVE
            lpForm1->FormReSize()
            Sizing=False
            IF lpForm1->OnResize>0 THEN
                lpForm1->OnResize(lpForm1)
                InvalidateRect(hWnd, NULL, True)
                RETURN 0
            END IF

        CASE WM_SIZE
            IF Sizing THEN RETURN 0
            IF (wParam=SIZE_MAXIMIZED) OR (wParam=SIZE_RESTORED) THEN
                lpForm1->FormReSize()
                IF lpForm1->OnResize>0 THEN
                    lpForm1->OnResize(lpForm1)
                    RETURN 0
                END IF
            END IF

        CASE WM_CLOSE

        CASE WM_DESTROY
            KillAllTimers()
            PostQuitMessage(0)

    END SELECT

    RETURN DefWindowProc(hWnd, wMsg, wParam, lParam)
END FUNCTION

'-------------------------------------------------------------------------------
' Create new window class for project
'-------------------------------------------------------------------------------
FUNCTION CreateWindClass(hInst AS HINSTANCE) AS integer
DIM AS integer Result
DIM AS WNDCLASSEX wc

WITH wc
    .cbSize = SizeOf(WNDCLASSEX)
    .style = CS_HREDRAW OR CS_VREDRAW
    .lpfnWndProc = @MainWndProc
    .hInstance = hInst
    .hIcon = LoadIcon(0, IDI_APPLICATION)
    .hCursor = LoadCursor(0, IDC_ARROW)
    .hbrBackground = GetSysColorBrush(COLOR_BTNFACE)
    .lpszClassName = @"RQClass"
    .hIconSm = .hIcon
END WITH

    Result = RegisterClassEx(@wc)
    IF Result = 0 THEN ShowMessage("Register class error")
    RETURN Result
END FUNCTION

'-------------------------------------------------------------------------------
' Main startup code
'-------------------------------------------------------------------------------
FUNCTION WinMain(hInst AS HINSTANCE, hPrevInstance AS HINSTANCE, _
                            szCmdLine AS string, iCmdShow AS integer) AS integer
DIM AS integer Result=0

    IF CreateWindClass(hInst) = 0 THEN RETURN 0

    ConfigureComponents()
    CreateComponents()
    InitSystemTimers(lpForm1->Handle)
    HookTheMouse()

    lpForm1->Center                         ' You may wish to delete this line

    IF lpForm1->OnShow>0 THEN lpForm1->OnShow(lpForm1)

    Result = lpForm1->ShowModal             ' Show form & call default message handler

    ' You may want to change the above line to lpForm1->Show, and then provide
    ' your own function that, amongst other things, handles all window messages.

    UnHookTheMouse()

    RETURN Result
END FUNCTION

'-------------------------------------------------------------------------------
' *** MAIN ENTRY POINT ***
'-------------------------------------------------------------------------------
END WinMain(Application.hInstance, NULL, Command(0), SW_NORMAL)


'-------------------------------------------------------------------------------
' Code for the component events
'-------------------------------------------------------------------------------
' #POPUP# 
SUB PopupMenuSelect(Index AS integer, Sender AS QFORM)
    SELECT CASE Index
        CASE TEXT_COPY:     ShowMessage("PopupMenu #" & Index)
        CASE TEXT_CUT:      ShowMessage("PopupMenu #" & Index)
        CASE TEXT_PASTE:    ShowMessage("PopupMenu #" & Index)
        CASE TEXT_SEL_ALL:  ShowMessage("PopupMenu #" & Index)
        CASE POPUP_ABOUT:   ShowAbout()
    END SELECT
END SUB

' #MENU# 
SUB Menu_Select(Index AS integer)
    SELECT CASE Index
        CASE MENUID_FILE_OPEN: TestOpenDialog()
        CASE MENUID_FILE_SAVE: TestSaveDialog()
        CASE MENUID_FILE_EXIT: Done()
        CASE MENUID_FONT_DLG: TestFontDialog()
        CASE MENUID_COLOR_DLG: TestColorDialog()
        CASE MENUID_PRNSET_DLG: TestPageSetup()
        CASE MENUID_PRINT_DLG: TestPrintDialog()
        CASE MENUID_EDIT_CUT: ShowMessage("MainMenu #" & Index & " [Cut]")
        CASE MENUID_EDIT_COPY: ShowMessage("MainMenu #" & Index & " [Copy]")
        CASE MENUID_EDIT_PASTE: ShowMessage("MainMenu #" & Index & " [Paste]")
        CASE MENUID_EDIT_UNDO: ShowMessage("MainMenu #" & Index & " [Undo]")
        CASE MENUID_EDIT_REDO: ShowMessage("MainMenu #" & Index & " [Redo]")
        CASE MENUID_SEARCH_FINDPREV: ShowMessage("MainMenu #" & Index & " [Find Prev]")
        CASE MENUID_SEARCH_FINDNEXT: ShowMessage("MainMenu #" & Index & " [Find Next]")
        CASE MENUID_SEARCH_REPLACE: ShowMessage("MainMenu #" & Index & " [Replace]")
        CASE MENUID_ABOUT: ShowAbout()
    END SELECT
END SUB

SUB TestOpenDialog()
DIM AS QOPENDIALOG dlg
#IF 0
    ' using all defaults
    IF dlg.Execute THEN
        ShowMessage("Selected: "+dlg.Filename)
    END IF
#ELSE
    ' Set some properties
'    dlg.Left=20: dlg.Top=-1
    dlg.NewStyle=True                                ' New explorer style dialog
    dlg.Caption="MY OPEN DIALOG"
    dlg.Filter="Bas Files (*.bas *.bi *.inc)|*.bas;*.bi;*.inc|Text Files (*.txt)|*.txt|All Files (*.*)|*.*"
    dlg.FilterIndex=1
    dlg.InitialDir="C:\"
    IF dlg.Execute THEN
        ShowMessage("Selected: "+dlg.Filename)
    END IF
#ENDIF
END SUB

SUB TestSaveDialog()
DIM AS QSAVEDIALOG dlg

    ' Set some properties - default initial directory application.path
    dlg.Caption="MY SAVE DIALOG"
    dlg.Filter="Text Files (*.txt)|*.txt|All Files (*.*)|*.*"
    dlg.FilterIndex=1
'    dlg.Left=20                                        ' Positioning if NewStyle=False
'    dlg.NewStyle=True                                  ' New explorer style dialog
    IF dlg.Execute THEN
        ShowMessage("Save file name: "+dlg.Filename)
    END IF
END SUB

SUB TestFontDialog()
DIM AS QFONTDIALOG dlg

'    dlg.Left=50: dlg.Top=50                             ' Preset to override defaults
    dlg.Caption = "MY FONT DIALOG (changes to QLABEL)"
    dlg.GetFont(@lpLabel1->Font)
    IF dlg.Execute() THEN
        dlg.SetFont(@lpLabel1->Font)                    ' Update the label font
    END IF
END SUB

SUB TestColorDialog()
DIM AS QCOLORDIALOG dlg

    ' Some presets to override defaults
    dlg.Caption = "MY COLOR DIALOG"
    dlg.Color=&hff0000                              ' Initial color
    dlg.Colors(0)=&hbf: dlg.Colors(1)=&hbf0000      ' Some pre-set defined colors
    dlg.Colors(2)=&h7f00: dlg.Colors(3)=&hbfbf
'    dlg.Left=20
    dlg.Style=cdFullOpen
    IF dlg.Execute THEN
        ShowMessage("You chose &h"+HEX(dlg.Color))
    END IF
END SUB

#DEFINE CRLF (CHR(13)+CHR(10))

SUB TestPageSetup()
DIM AS QPAGESETUPDIALOG dlg
DIM AS string tstr
DIM AS double ttt=91.6834

    dlg.Caption="PAGE SETUP DIALOG"
'    dlg.Metric=False
'    dlg.Left=20                        ' Vertical centered or add dlg.Top=20
    IF dlg.Execute THEN
        tstr="Printer: "+dlg.PrinterName+CRLF+CRLF
        IF dlg.Metric THEN
            tstr=tstr+"Paper size (mm):"+CRLF
        ELSE
            tstr=tstr+"Paper size (inch):"+CRLF
        END IF
        tstr=tstr+"    Width: "+STR(dlg.PaperWidth)+CRLF+"    Height: "+STR(dlg.PaperHeight)+CRLF+CRLF
        IF dlg.Metric THEN
            tstr=tstr+"Margins (mm):"+CRLF
        ELSE
            tstr=tstr+"Margins (inch):"+CRLF
        END IF
        tstr=tstr+"    Left: "+STR(dlg.LeftMargin)+CRLF+"    Right: "+STR(dlg.RightMargin)+CRLF
        tstr=tstr+"    Top: "+STR(dlg.TopMargin)+CRLF+"    Bottom: "+STR(dlg.BottomMargin)
        ShowMessage(tstr)
    END IF
END SUB

SUB TestPrintDialog()
DIM AS QPRINTDIALOG dlg

'    dlg.SetupDialog=True                              ' Default False - A Print Dialog
    dlg.Caption="MY PRINT DIALOG"
    dlg.AllowToFile=False
    dlg.AllowToNetwork=False                           ' Add network printers?
'    dlg.Left=20
    IF dlg.Execute THEN
        ShowMessage("(Dummy) start printing to Printer: "+dlg.PrinterName)
#IF 0
        ' Do the TEXT print...
        DIM AS integer prnOut=FreeFile()
        IF OPEN lpt("LPT:"+dlg.PrinterName+",EMU=TTY", FOR OUTPUT, AS #prnOut) THEN
            PRINT #prnOut, "Hello World!"
            CLOSE #prnOut
            ShowMessage("Printing now to "+dlg.PrinterName)
        ELSE
            ShowMessage("Error: Printer open failed")
        END IF
#ENDIF
    END IF
END SUB

SUB ShowAbout
DIM AS string tstr

    tstr="Demo of a QMainMenu with multiple sub-menus plus"+CRLF
    tstr=tstr+"a QPopupMenu is attached to the Form."+CRLF+CRLF
    tstr=tstr+"Several dialogs are also demonstrated."
    ShowMessage(tstr)
END SUB

SUB Done()
    Application.Terminate
END SUB

